# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/friendlly',
                'doc_host': 'https://xl0.github.io',
                'git_url': 'https://github.com/xl0/friendlly',
                'lib_path': 'friendlly'},
  'syms': { 'friendlly.display': { 'friendlly.display.PassthroughCapturer': ('display.html#passthroughcapturer', 'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.__init__': ( 'display.html#passthroughcapturer.__init__',
                                                                                       'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.clear_output': ( 'display.html#passthroughcapturer.clear_output',
                                                                                           'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.get_outputs': ( 'display.html#passthroughcapturer.get_outputs',
                                                                                          'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.publish': ( 'display.html#passthroughcapturer.publish',
                                                                                      'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.start': ( 'display.html#passthroughcapturer.start',
                                                                                    'friendlly/display.py'),
                                   'friendlly.display.PassthroughCapturer.stop': ( 'display.html#passthroughcapturer.stop',
                                                                                   'friendlly/display.py')},
            'friendlly.magic_cell': { 'friendlly.magic_cell.CellChat': ('magic_cell.html#cellchat', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.__init__': ( 'magic_cell.html#cellchat.__init__',
                                                                                  'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.delete_markdown': ( 'magic_cell.html#cellchat.delete_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.go': ('magic_cell.html#cellchat.go', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.update_code': ( 'magic_cell.html#cellchat.update_code',
                                                                                     'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.CellChat.update_markdown': ( 'magic_cell.html#cellchat.update_markdown',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.cells_to_messages': ( 'magic_cell.html#cells_to_messages',
                                                                                  'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.display_data_to_messages': ( 'magic_cell.html#display_data_to_messages',
                                                                                         'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.fr_cell': ('magic_cell.html#fr_cell', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.handle_config': ('magic_cell.html#handle_config', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.make_cell_from_captured': ( 'magic_cell.html#make_cell_from_captured',
                                                                                        'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.merge_messages': ('magic_cell.html#merge_messages', 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.parse_above_cell': ( 'magic_cell.html#parse_above_cell',
                                                                                 'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.parse_cell_line': ( 'magic_cell.html#parse_cell_line',
                                                                                'friendlly/magic_cell.py'),
                                      'friendlly.magic_cell.strip_comments': ('magic_cell.html#strip_comments', 'friendlly/magic_cell.py')},
            'friendlly.magic_line': { 'friendlly.magic_line.fr_line': ('magic_line.html#fr_line', 'friendlly/magic_line.py'),
                                      'friendlly.magic_line.parse_cell': ('magic_line.html#parse_cell', 'friendlly/magic_line.py')},
            'friendlly.utils.capture_display': { 'friendlly.utils.capture_display.PassthroughCapturer': ( 'utils.capture_display.html#passthroughcapturer',
                                                                                                          'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.__init__': ( 'utils.capture_display.html#passthroughcapturer.__init__',
                                                                                                                   'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.clear_output': ( 'utils.capture_display.html#passthroughcapturer.clear_output',
                                                                                                                       'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.get_outputs': ( 'utils.capture_display.html#passthroughcapturer.get_outputs',
                                                                                                                      'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.publish': ( 'utils.capture_display.html#passthroughcapturer.publish',
                                                                                                                  'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.start': ( 'utils.capture_display.html#passthroughcapturer.start',
                                                                                                                'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.PassthroughCapturer.stop': ( 'utils.capture_display.html#passthroughcapturer.stop',
                                                                                                               'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.display_to_messages': ( 'utils.capture_display.html#display_to_messages',
                                                                                                          'friendlly/utils/capture_display.py'),
                                                 'friendlly.utils.capture_display.svg_to_png': ( 'utils.capture_display.html#svg_to_png',
                                                                                                 'friendlly/utils/capture_display.py')},
            'friendlly.utils.capture_io': { 'friendlly.utils.capture_io.TeeIO': ( 'utils.capture_io.html#teeio',
                                                                                  'friendlly/utils/capture_io.py'),
                                            'friendlly.utils.capture_io.TeeIO.__getattr__': ( 'utils.capture_io.html#teeio.__getattr__',
                                                                                              'friendlly/utils/capture_io.py'),
                                            'friendlly.utils.capture_io.TeeIO.__init__': ( 'utils.capture_io.html#teeio.__init__',
                                                                                           'friendlly/utils/capture_io.py'),
                                            'friendlly.utils.capture_io.TeeIO.__setattr__': ( 'utils.capture_io.html#teeio.__setattr__',
                                                                                              'friendlly/utils/capture_io.py'),
                                            'friendlly.utils.capture_io.TeeIO.getvalue': ( 'utils.capture_io.html#teeio.getvalue',
                                                                                           'friendlly/utils/capture_io.py'),
                                            'friendlly.utils.capture_io.TeeIO.write': ( 'utils.capture_io.html#teeio.write',
                                                                                        'friendlly/utils/capture_io.py')},
            'friendlly.utils.capture_trace': { 'friendlly.utils.capture_trace.SaveTraceback': ( 'utils.capture_trace.html#savetraceback',
                                                                                                'friendlly/utils/capture_trace.py'),
                                               'friendlly.utils.capture_trace.SaveTraceback.__init__': ( 'utils.capture_trace.html#savetraceback.__init__',
                                                                                                         'friendlly/utils/capture_trace.py'),
                                               'friendlly.utils.capture_trace.SaveTraceback._showtraceback': ( 'utils.capture_trace.html#savetraceback._showtraceback',
                                                                                                               'friendlly/utils/capture_trace.py'),
                                               'friendlly.utils.capture_trace.SaveTraceback.register': ( 'utils.capture_trace.html#savetraceback.register',
                                                                                                         'friendlly/utils/capture_trace.py'),
                                               'friendlly.utils.capture_trace.SaveTraceback.unregister': ( 'utils.capture_trace.html#savetraceback.unregister',
                                                                                                           'friendlly/utils/capture_trace.py'),
                                               'friendlly.utils.capture_trace.strip_junk': ( 'utils.capture_trace.html#strip_junk',
                                                                                             'friendlly/utils/capture_trace.py')},
            'friendlly.utils.cell_cbs': { 'friendlly.utils.cell_cbs.ExecCBs': ( 'utils.cell_cbs.html#execcbs',
                                                                                'friendlly/utils/cell_cbs.py'),
                                          'friendlly.utils.cell_cbs.ExecCBs.__init__': ( 'utils.cell_cbs.html#execcbs.__init__',
                                                                                         'friendlly/utils/cell_cbs.py'),
                                          'friendlly.utils.cell_cbs.ExecCBs.post_callback': ( 'utils.cell_cbs.html#execcbs.post_callback',
                                                                                              'friendlly/utils/cell_cbs.py'),
                                          'friendlly.utils.cell_cbs.ExecCBs.pre_callback': ( 'utils.cell_cbs.html#execcbs.pre_callback',
                                                                                             'friendlly/utils/cell_cbs.py'),
                                          'friendlly.utils.cell_cbs.ExecCBs.register': ( 'utils.cell_cbs.html#execcbs.register',
                                                                                         'friendlly/utils/cell_cbs.py'),
                                          'friendlly.utils.cell_cbs.ExecCBs.unregister': ( 'utils.cell_cbs.html#execcbs.unregister',
                                                                                           'friendlly/utils/cell_cbs.py')},
            'friendlly.utils.misc': { 'friendlly.utils.misc.GenericEncoder': ('utils.misc.html#genericencoder', 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.GenericEncoder.default': ( 'utils.misc.html#genericencoder.default',
                                                                                       'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict': ('utils.misc.html#nict', 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict.__delattr__': ( 'utils.misc.html#nict.__delattr__',
                                                                                 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict.__dir__': ('utils.misc.html#nict.__dir__', 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict.__getattr__': ( 'utils.misc.html#nict.__getattr__',
                                                                                 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict.__init__': ('utils.misc.html#nict.__init__', 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict.__setattr__': ( 'utils.misc.html#nict.__setattr__',
                                                                                 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.nict._to_dict': ('utils.misc.html#nict._to_dict', 'friendlly/utils/misc.py'),
                                      'friendlly.utils.misc.safe_int': ('utils.misc.html#safe_int', 'friendlly/utils/misc.py')},
            'friendlly.utils.notebook': { 'friendlly.utils.notebook.detect_environment': ( 'utils.notebook.html#detect_environment',
                                                                                           'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.extract_vscode_path': ( 'utils.notebook.html#extract_vscode_path',
                                                                                            'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.inject_js': ( 'utils.notebook.html#inject_js',
                                                                                  'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_add_cell': ( 'utils.notebook.html#nbclassic_add_cell',
                                                                                           'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_delete_cell': ( 'utils.notebook.html#nbclassic_delete_cell',
                                                                                              'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_execute_cell': ( 'utils.notebook.html#nbclassic_execute_cell',
                                                                                               'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_get_index': ( 'utils.notebook.html#nbclassic_get_index',
                                                                                            'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_patch_kernel': ( 'utils.notebook.html#nbclassic_patch_kernel',
                                                                                               'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_render_cell': ( 'utils.notebook.html#nbclassic_render_cell',
                                                                                              'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.nbclassic_update_cell': ( 'utils.notebook.html#nbclassic_update_cell',
                                                                                              'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.update_code_next': ( 'utils.notebook.html#update_code_next',
                                                                                         'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.update_code_self': ( 'utils.notebook.html#update_code_self',
                                                                                         'friendlly/utils/notebook.py'),
                                          'friendlly.utils.notebook.vscode_get_cells': ( 'utils.notebook.html#vscode_get_cells',
                                                                                         'friendlly/utils/notebook.py')}}}
